#include <gtk/gtk.h>
#include <gdk/gdkscreen.h>
#include <cairo.h>
 
/*
 * Author:
 *	Marc Di COLO (marc.dicolo@gmail.com
 *
 * Licence : 
 *	GPLV3 (SPDX)
 *
 * Object/Usage :
 *	Help (a bit) the collimation of a Shmidt Cassegrain Telescope.
 *	Resize the window to resize the outer ring.
 *	Left click/Right click to shrink/enlarge the inner ring.
 *	middle button changes the color of the rings (R V B).
 *
 * Others:
 *	Basic skills in compilation needed ... 
 *	build with : 
		gcc $(pkg-config --libs --cflags gtk+-2.0) main.c -o sct-colim
 */
 
static void display_changed(GtkWidget *widget, GdkScreen *old_screen, gpointer user_data);
static gboolean expose(GtkWidget *widget, GdkEventExpose *event, gpointer user_data);
static void clicked(GtkWindow *win, GdkEventButton *event, gpointer user_data);

int inner=96;
int R=1, V=0, B=0;
 
int main(int argc, char **argv)
{
    gtk_init(&argc, &argv);
 
    GtkWidget *window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title(GTK_WINDOW(window), "SCT Collimation");
	gtk_window_set_default_size(GTK_WINDOW(window), 384, 384);
	gtk_window_set_position (window, GTK_WIN_POS_CENTER);
	gtk_window_set_keep_above(GTK_WINDOW(window), TRUE);
    g_signal_connect(G_OBJECT(window), "delete-event", gtk_main_quit, NULL);
 
 
    /* We want to draw the window background ourself so we can keep it transparent. */
    gtk_widget_set_app_paintable(window, TRUE);
 
	/* The two system events we need ...*/ 
    g_signal_connect(G_OBJECT(window), "expose-event", G_CALLBACK(expose), NULL);
    g_signal_connect(G_OBJECT(window), "screen-changed", G_CALLBACK(display_changed), NULL);
 
    gtk_widget_add_events(window, GDK_BUTTON_PRESS_MASK);
	
    g_signal_connect(G_OBJECT(window), "button-press-event", G_CALLBACK(clicked), NULL);
    g_signal_connect(G_OBJECT(window), "check-resize", G_CALLBACK(expose), NULL);

    /* Initialize the current display */
    display_changed(window, NULL, NULL);
 
    gtk_widget_show_all(window);
    gtk_main();
 
    return 0;
}
 
 
/* Fall back if X server does not support alpha channels. */
gboolean supports_alpha = FALSE;
 
static void display_changed(GtkWidget *widget, GdkScreen *old_screen, gpointer userdata)
{
    GdkScreen *screen = gtk_widget_get_screen(widget);
    GdkColormap *colormap = gdk_screen_get_rgba_colormap(screen);
 
    if (!colormap)
    {
        printf("supports_alpha = FALSE\n");
        colormap = gdk_screen_get_rgb_colormap(screen);
        supports_alpha = FALSE;
    }
    else
    {
        printf("supports_alpha = TRUE\n");
        supports_alpha = TRUE;
    }
 
    gtk_widget_set_colormap(widget, colormap);
}
 
/* draws our circles */
static gboolean expose(GtkWidget *widget, GdkEventExpose *event, gpointer userdata)
{
    cairo_t *cr = gdk_cairo_create(widget->window);
 
    if (supports_alpha)
        cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.0); /* transparent */
    else
        cairo_set_source_rgb (cr, 1.0, 1.0, 1.0); /* opaque */
 
    /* draw the background */
    cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
    cairo_paint (cr);
 
    /* draw a circle */
    int width=0, height=0;
    gtk_window_get_size(GTK_WINDOW(widget), &width, &height);
    cairo_set_source_rgba(cr, R, V, B, 0.8);
	cairo_set_line_width(cr, 2);
    cairo_arc(cr, width / 2, height / 2, (width < height ? width : height) / 2 - 48 , 0, 2 * 3.14);
    cairo_arc(cr, width / 2, height / 2, (width < height ? width : height) / 2 - inner , 0, 2 * 3.14);

	/* draw horizontal line */
	cairo_move_to(cr, 0, height / 2);
	cairo_line_to(cr, width, height / 2);

	/* draw vertical line */
	cairo_move_to(cr, width / 2, 0);
	cairo_line_to(cr, width/2, height);
	
    // cairo_fill(cr);
    cairo_stroke (cr);

    cairo_destroy(cr);
    return FALSE;
}
 
/* Use buttons to resize the inner ring and change the color */
static void clicked(GtkWindow *win, GdkEventButton *event, gpointer user_data)
{
	GdkEvent * testButton = gtk_get_current_event();
    guint typeClic=(*testButton).button.button ;

    if(typeClic==1){				/* Left click, we shrink the inner ring */
        inner+=2;
    }

    else if( typeClic ==2){			/* Middle click, we change the rings color */
        if		(R==1){	R=0; V=1; B=0; }
        else if	(V==1){	R=0; V=0; B=1; }
        else if	(B==1){	R=1; V=0; B=0; }
	}

    else if( typeClic ==3){			/* Right clic, we enlarge the inner ring */
        inner-=2;
    }

	expose( win, NULL, NULL);
}

